@@ -11,7 +11,7 @@
 from __future__ import print_function
 from __future__ import unicode_literals
 from __future__ import division
-from __future__ import absolute_import    
+from __future__ import absolute_import
 #from pynlpl.common import u
 import sys
 if sys.version < '3':
@@ -21,21 +21,22 @@ if sys.version < '3':
 else:
     stderr = sys.stderr
     stdout = sys.stdout
-    
+
 from pynlpl.statistics import FrequencyList, product
 from pynlpl.textprocessors import Windower
+import math
 import io
 
 
 
 class SimpleLanguageModel:
-    """This is a very simple unsmoothed language model"""
-    
+    """This is a simple unsmoothed language model. This class can both hold and compute the model."""
+
     def __init__(self, n=2, casesensitive = True, beginmarker = "<begin>", endmarker = "<end>"):
         self.casesensitive = casesensitive
         self.freqlistN = FrequencyList(None, self.casesensitive)
         self.freqlistNm1 = FrequencyList(None, self.casesensitive)
-        
+
         assert isinstance(n,int) and n >= 2
         self.n = n
         self.beginmarker = beginmarker
@@ -54,7 +55,7 @@ class SimpleLanguageModel:
         for ngram in Windower(sentence,self.n, self.beginmarker, self.endmarker):
             self.freqlistN.count(ngram)
         for ngram in Windower(sentence,self.n-1, self.beginmarker, self.endmarker):
-            self.freqlistNm1.count(ngram)  
+            self.freqlistNm1.count(ngram)
 
 
     def load(self, filename):
@@ -76,7 +77,7 @@ class SimpleLanguageModel:
                     elif line[:12] == 'beginmarker=':
                         self.beginmarker = line[12:]
                     elif line[:10] == 'endmarker=':
-                        self.endmarker = line[10:]   
+                        self.endmarker = line[10:]
                     elif line[:10] == 'sentences=':
                         self.sentences = int(line[10:])
                     elif line[:14] == 'casesensitive=':
@@ -129,7 +130,7 @@ class SimpleLanguageModel:
 
     def scoresentence(self, sentence):
         return product([self[x] for x in Windower(sentence, self.n, self.beginmarker, self.endmarker)])
-            
+
 
     def __getitem__(self, ngram):
         assert len(ngram) == self.n
@@ -138,10 +139,101 @@ class SimpleLanguageModel:
 
         if (self.beginmarker and nm1gram == self._begingram) or (self.endmarker and nm1gram == self._endgram):
             return self.freqlistN[ngram] / float(self.sentences)
-        else:   
+        else:
             return self.freqlistN[ngram] / float(self.freqlistNm1[nm1gram])
 
 
+class ARPALanguageModel(object):
+    """Full back-off language model, loaded from file in ARPA format. This class does not build the model but allows you to use a pre-computed one. You can use the tool ngram-count from for instance SRILM to actually build the model. """
+
+    def __init__(self, filename, encoding = 'utf-8', encoder=None, base_e=True, debug=False):
+        self.ngrams = {}
+        self.backoff = {}
+        self.total = {}
+        self.base_e = base_e
+        self.debug = False
+
+        if encoder == None:
+            self.encoder = lambda x: x
+        else:
+            self.encoder = encoder
+
+        with io.open(filename,'r',encoding) as f:
+            for line in f:
+                line = line.strip()
+                if line == '\\data\\':
+                    order = 0
+                elif line[0] == '\\' and line[-1] == ':':
+                    for i in range(1,10):
+                        if line == '\\' + str(i) + '-grams:':
+                            order = i
+                elif line:
+                    if order == 0:
+                        if line[0:6] == "ngram":
+                            n = int(line[6])
+                            v = int(line[8])
+                            self.total[n] = v
+                    elif order > 0:
+                        fields = line.split('\t')
+                        if base_e:
+                            logprob = float(fields[0]) * math.log(10)   # * log(10) does log10 to log_e conversion
+                        else:
+                            logprob = float(fields[0])
+                        ngram = encoder(tuple(fields[1].split()))
+                        self.ngrams[ngram] = logprob
+                        if len(fields) > 2:
+                            if base_e:
+                                backoffprob = float(fields[2]) * math.log(10)
+                            else:
+                                backoffprob = float(fields[2])
+                            self.backoff[ngram] = backoffprob
+                            if self.debug:
+                                print("Adding to LM: " + str(ngram) + "\t" << str(logprob) + "\t" + str(backoffprob), file=stderr)
+                        elif self.debug:
+                            print("Adding to LM: " + str(ngram) + "\t" << str(logprob), file=stderr)
+                    elif self.debug:
+                        print("Unable to parse ARPA LM line: " + line, file=stderr)
+
+        self.order = order
+
+    def score(self, data, history=None):
+        result = 0
+        for word in data:
+            result += self.scoreword(word, history)
+            if history:
+                history += (word,)
+            else:
+                history = (word,)
+        return result
+
+    def scoreword(self, word, history=None):
+        if history:
+            lookup = history + word
+        else:
+            lookup = word
+
+        if len(lookup) > self.order:
+            lookup = lookup[-self.order:]
+
+        try:
+            return self.ngrams[lookup]
+        except KeyError:
+
+            #not found, back off
+            if not history:
+                raise KeyError("Word " + str(word) + " not found. And no history specified")
+
+            try:
+                backoffweight = self.backoff[history]
+            except KeyError:
+                backoffweight = 0 #backoff weight will be 0 if not found
+            return backoffweight + self.scoreword(word, history[1:])
+
+
+
+    def __len__(self):
+        return len(self.ngrams)
+
 
 
 
